@@ -19,6 +19,9 @@
  *
  */
 
+#include "dirent_lookup.h"
+#include "zim_types.h"
+#include <memory>
 #define CHUNK_SIZE 1024
 #include "fileimpl.h"
 #include <zim/error.h>
@@ -34,9 +37,9 @@
 #include <numeric>
 #include "config.h"
 #include "log.h"
-#include "envvalue.h"
 #include "md5.h"
 #include "tools.h"
+#include "fileheader.h"
 
 log_define("zim.file.impl")
 
@@ -184,11 +187,12 @@ private: // data
     : zimFile(_zimFile),
       zimReader(makeFileReader(zimFile)),
       direntReader(new DirentReader(zimReader)),
-      clusterCache(envValue("ZIM_CLUSTERCACHE", CLUSTER_CACHE_SIZE)),
-      m_newNamespaceScheme(false),
+      clusterCache(CLUSTER_CACHE_SIZE),
       m_hasFrontArticlesIndex(true),
       m_startUserEntry(0),
-      m_endUserEntry(0)
+      m_endUserEntry(0),
+      m_direntLookupCreated(false),
+      m_direntLookupSize(DIRENT_LOOKUP_CACHE_SIZE)
   {
     log_trace("read file \"" << zimFile->filename() << '"');
 
@@ -231,9 +235,24 @@ private: // data
 
     quickCheckForCorruptFile();
 
-    mp_titleDirentAccessor = getTitleAccessor("listing/titleOrdered/v1");
+    DirentLookup tmpDirentLookup(mp_pathDirentAccessor.get());
+
+    if (header.useNewNamespaceScheme()) {
+      const_cast<entry_index_t&>(m_startUserEntry) = tmpDirentLookup.getNamespaceRangeBegin('C');
+      const_cast<entry_index_t&>(m_endUserEntry) = tmpDirentLookup.getNamespaceRangeEnd('C');
+    } else {
+      const_cast<entry_index_t&>(m_endUserEntry) = getCountArticles();
+    }
+
+    auto result = tmpDirentLookup.find('X', "listing/titleOrdered/v1");
+    if (result.first) {
+      mp_titleDirentAccessor = getTitleAccessorV1(result.second);
+    }
 
     if (!mp_titleDirentAccessor) {
+      if (!header.hasTitleListingV0()) {
+        throw ZimFileFormatError("Zim file doesn't contain a title ordered index");
+      }
       offset_t titleOffset(header.getTitleIdxPos());
       zsize_t  titleSize(sizeof(entry_index_type)*header.getArticleCount());
       mp_titleDirentAccessor = getTitleAccessor(titleOffset, titleSize, "Title index table");
@@ -244,14 +263,9 @@ private: // data
     readMimeTypes();
   }
 
-  std::unique_ptr<IndirectDirentAccessor> FileImpl::getTitleAccessor(const std::string& path)
+  std::unique_ptr<IndirectDirentAccessor> FileImpl::getTitleAccessorV1(const entry_index_t idx)
   {
-    auto result = direntLookup().find('X', path);
-    if (!result.first) {
-      return nullptr;
-    }
-
-    auto dirent = mp_pathDirentAccessor->getDirent(result.second);
+    auto dirent = mp_pathDirentAccessor->getDirent(idx);
     auto cluster = getCluster(dirent->getClusterNumber());
     if (cluster->isCompressed()) {
       // This is a ZimFileFormatError.
@@ -260,7 +274,7 @@ private: // data
     }
     auto titleOffset = getClusterOffset(dirent->getClusterNumber()) + cluster->getBlobOffset(dirent->getBlobNumber());
     auto titleSize = cluster->getBlobSize(dirent->getBlobNumber());
-    return getTitleAccessor(titleOffset, titleSize, "Title index table" + path);
+    return getTitleAccessor(titleOffset, titleSize, "Title index v1");
   }
 
   std::unique_ptr<IndirectDirentAccessor> FileImpl::getTitleAccessor(const offset_t offset, const zsize_t size, const std::string& name)
@@ -282,11 +296,15 @@ private: // data
     //    in the call stack.
     // 2. With `glibc` an exceptional execution of `std::call_once` doesn't
     //    unlock the mutex associated with the `std::once_flag` object.
-    if ( !m_direntLookup ) {
+    if (!m_direntLookupCreated.load(std::memory_order_acquire)) {
       std::lock_guard<std::mutex> lock(m_direntLookupCreationMutex);
       if ( !m_direntLookup ) {
-        const auto cacheSize = envValue("ZIM_DIRENTLOOKUPCACHE", DIRENT_LOOKUP_CACHE_SIZE);
-        m_direntLookup.reset(new DirentLookup(mp_pathDirentAccessor.get(), cacheSize));
+        if (m_direntLookupSize == 0) {
+          m_direntLookup = std::make_unique<DirentLookup>(mp_pathDirentAccessor.get());
+        } else {
+          m_direntLookup = std::make_unique<FastDirentLookup>(mp_pathDirentAccessor.get(), m_direntLookupSize);
+        }
+        m_direntLookupCreated.store(true, std::memory_order_release);
       }
     }
     return *m_direntLookup;
@@ -311,7 +329,9 @@ private: // data
   offset_type FileImpl::getMimeListEndUpperLimit() const
   {
     offset_type result(header.getPathPtrPos());
-    result = std::min(result, header.getTitleIdxPos());
+    if (header.hasTitleListingV0()) {
+      result = std::min(result, header.getTitleIdxPos());
+    }
     result = std::min(result, header.getClusterPtrPos());
     if ( getCountArticles().v != 0 ) {
       // assuming that dirents are placed in the zim file in the same
@@ -358,14 +378,6 @@ private: // data
 
       p = zp+1;
     }
-
-    const_cast<bool&>(m_newNamespaceScheme) = header.getMinorVersion() >= 1;
-    if (m_newNamespaceScheme) {
-      const_cast<entry_index_t&>(m_startUserEntry) = getNamespaceBeginOffset('C');
-      const_cast<entry_index_t&>(m_endUserEntry) = getNamespaceEndOffset('C');
-    } else {
-      const_cast<entry_index_t&>(m_endUserEntry) = getCountArticles();
-    }
   }
 
   FileImpl::FindxResult FileImpl::findx(char ns, const std::string& path)
@@ -553,11 +565,11 @@ private: // data
 
     struct zim_MD5_CTX md5ctx;
     zim_MD5Init(&md5ctx);
-    
+
     unsigned char ch[CHUNK_SIZE];
     offset_type checksumPos = header.getChecksumPos();
     offset_type toRead = checksumPos;
-    
+
     for(auto part = zimFile->begin();
         part != zimFile->end();
         part++) {
@@ -567,7 +579,7 @@ private: // data
         zim_MD5Update(&md5ctx, ch, CHUNK_SIZE);
         toRead-=CHUNK_SIZE;
       }
-      
+
       // Previous read was good, so we have exited the previous `while` because
       // `toRead<CHUNK_SIZE`. Let's try to read `toRead` chars and process them later.
       // Else, the previous `while` exited because we didn't succeed to read
@@ -576,12 +588,12 @@ private: // data
       if(stream.good()){
         stream.read(reinterpret_cast<char*>(ch),toRead);
       }
-      
+
       // It updates the checksum with the remaining amount of data when we
       // reach the end of the file or part
       zim_MD5Update(&md5ctx, ch, stream.gcount());
       toRead-=stream.gcount();
-    
+
       if (stream.bad()) {
         perror("error while reading file");
         return false;
@@ -743,12 +755,19 @@ bool checkTitleListing(const IndirectDirentAccessor& accessor, entry_index_type
   bool FileImpl::checkTitleIndex() {
     const entry_index_type articleCount = getCountArticles().v;
 
-    offset_t titleOffset(header.getTitleIdxPos());
-    zsize_t  titleSize(sizeof(entry_index_type)*header.getArticleCount());
-    auto titleDirentAccessor = getTitleAccessor(titleOffset, titleSize, "Full Title index table");
-    auto ret = checkTitleListing(*titleDirentAccessor, articleCount);
+    auto ret = true;
+    if (header.hasTitleListingV0()) {
+      offset_t titleOffset(header.getTitleIdxPos());
+      zsize_t  titleSize(sizeof(entry_index_type)*header.getArticleCount());
+      auto titleDirentAccessor = getTitleAccessor(titleOffset, titleSize, "Full Title index table");
+      ret = checkTitleListing(*titleDirentAccessor, articleCount);
+    }
 
-    titleDirentAccessor = getTitleAccessor("listing/titleOrdered/v1");
+    auto titleDirentAccessor = std::unique_ptr<IndirectDirentAccessor>();
+    auto result = direntLookup().find('X', "listing/titleOrdered/v1");
+    if (result.first) {
+      titleDirentAccessor = getTitleAccessorV1(result.second);
+    }
     if (titleDirentAccessor) {
       ret &= checkTitleListing(*titleDirentAccessor, articleCount);
     }
@@ -770,4 +789,32 @@ bool checkTitleListing(const IndirectDirentAccessor& accessor, entry_index_type
     return true;
   }
 
+
+  size_t FileImpl::getClusterCacheMaxSize() const {
+    return clusterCache.getMaxSize();
+  }
+  size_t FileImpl::getClusterCacheCurrentSize() const {
+    return clusterCache.getCurrentSize();
+  }
+  void FileImpl::setClusterCacheMaxSize(size_t nbClusters) {
+    clusterCache.setMaxSize(nbClusters);
+  }
+
+  size_t FileImpl::getDirentCacheMaxSize() const {
+    return mp_pathDirentAccessor->getMaxCacheSize();
+  }
+  size_t FileImpl::getDirentCacheCurrentSize() const {
+    return mp_pathDirentAccessor->getCurrentCacheSize();
+  }
+  void FileImpl::setDirentCacheMaxSize(size_t nbDirents) {
+    mp_pathDirentAccessor->setMaxCacheSize(nbDirents);
+  }
+
+  size_t FileImpl::getDirentLookupCacheMaxSize() const {
+    if (!m_direntLookupCreated.load(std::memory_order_acquire)) {
+      return m_direntLookupSize;
+    } else {
+      return m_direntLookup->getSize();
+    }
+  }
 }
