@@ -22,6 +22,7 @@
 #ifndef ZIM_FILEIMPL_H
 #define ZIM_FILEIMPL_H
 
+#include <atomic>
 #include <string>
 #include <vector>
 #include <memory>
@@ -56,7 +57,6 @@ namespace zim
       typedef std::shared_ptr<const Cluster> ClusterHandle;
       ConcurrentCache<cluster_index_type, ClusterHandle> clusterCache;
 
-      const bool m_newNamespaceScheme;
       const bool m_hasFrontArticlesIndex;
       const entry_index_t m_startUserEntry;
       const entry_index_t m_endUserEntry;
@@ -76,9 +76,12 @@ namespace zim
         }
       };
 
-      using DirentLookup = zim::FastDirentLookup<DirentLookupConfig>;
+      using DirentLookup = zim::DirentLookup<DirentLookupConfig>;
+      using FastDirentLookup = zim::FastDirentLookup<DirentLookupConfig>;
       mutable std::unique_ptr<DirentLookup> m_direntLookup;
       mutable std::mutex m_direntLookupCreationMutex;
+      mutable std::atomic_bool m_direntLookupCreated;
+      size_t m_direntLookupSize;
 
 
       struct ByTitleDirentLookupConfig
@@ -109,7 +112,7 @@ namespace zim
       const std::string& getFilename() const   { return zimFile->filename(); }
       const Fileheader& getFileheader() const  { return header; }
       zsize_t getFilesize() const;
-      bool hasNewNamespaceScheme() const { return m_newNamespaceScheme; }
+      bool hasNewNamespaceScheme() const { return header.useNewNamespaceScheme(); }
       bool hasFrontArticlesIndex() const { return m_hasFrontArticlesIndex; }
 
       FileCompound::PartRange getFileParts(offset_t offset, zsize_t size);
@@ -149,11 +152,20 @@ namespace zim
       bool is_multiPart() const;
 
       bool checkIntegrity(IntegrityCheck checkType);
+
+      size_t getClusterCacheMaxSize() const;
+      size_t getClusterCacheCurrentSize() const;
+      void setClusterCacheMaxSize(size_t nbClusters);
+      size_t getDirentCacheMaxSize() const;
+      size_t getDirentCacheCurrentSize() const;
+      void setDirentCacheMaxSize(size_t nbDirents);
+      size_t getDirentLookupCacheMaxSize() const;
+      void setDirentLookupCacheMaxSize(size_t nbRanges) { m_direntLookupSize = nbRanges; };
   private:
       explicit FileImpl(std::shared_ptr<FileCompound> zimFile);
       FileImpl(std::shared_ptr<FileCompound> zimFile, offset_t offset, zsize_t size);
 
-      std::unique_ptr<IndirectDirentAccessor> getTitleAccessor(const std::string& path);
+      std::unique_ptr<IndirectDirentAccessor> getTitleAccessorV1(const entry_index_t idx);
       std::unique_ptr<IndirectDirentAccessor> getTitleAccessor(const offset_t offset, const zsize_t size, const std::string& name);
 
       void prepareArticleListByCluster() const;
