@@ -29,6 +29,7 @@
 #include "config.h"
 
 #include <zim/item.h>
+#include "zim/writer/item.h"
 
 #if defined(ENABLE_XAPIAN)
 namespace Xapian {
@@ -70,6 +71,11 @@ namespace zim {
     return count;
   }
 
+  namespace writer {
+    class Dirent;
+    bool isFrontArticle(const Dirent* dirent, const Hints& hints);
+  }
+
 // Xapian based tools
 #if defined(ENABLE_XAPIAN)
   std::string LIBZIM_PRIVATE_API removeAccents(const std::string& text);
