@@ -35,6 +35,7 @@
 
 const int NUM_OF_TEST2_RECORDS = 100;
 const unsigned int TEST2_CACHE_CAPACITY = 50u;
+const unsigned int TEST2_CACHE_CAPACITY_SMALL = 10u;
 
 TEST(CacheTest, SimplePut) {
     zim::lru_cache<int, int> cache_lru(1);
@@ -78,6 +79,19 @@ TEST(CacheTest, DropValue) {
     EXPECT_FALSE(cache_lru.drop(7));
 }
 
+#define EXPECT_RANGE_MISSING_FROM_CACHE(CACHE, START, END) \
+for (unsigned i = START; i  < END; ++i)  {                 \
+  EXPECT_FALSE(CACHE.exists(i));                           \
+}
+
+#define EXPECT_RANGE_FULLY_IN_CACHE(CACHE, START, END, VALUE_KEY_RATIO) \
+for (unsigned i = START; i  < END; ++i)  {                              \
+  EXPECT_TRUE(CACHE.exists(i));                                         \
+  EXPECT_EQ(i*VALUE_KEY_RATIO, cache_lru.get(i));                       \
+}
+
+
+
 TEST(CacheTest1, KeepsAllValuesWithinCapacity) {
     zim::lru_cache<int, int> cache_lru(TEST2_CACHE_CAPACITY);
 
@@ -85,19 +99,39 @@ TEST(CacheTest1, KeepsAllValuesWithinCapacity) {
         cache_lru.put(i, i);
     }
 
-    for (unsigned i = 0; i < NUM_OF_TEST2_RECORDS - TEST2_CACHE_CAPACITY; ++i) {
-        EXPECT_FALSE(cache_lru.exists(i));
-    }
+    EXPECT_RANGE_MISSING_FROM_CACHE(cache_lru, 0, (NUM_OF_TEST2_RECORDS - TEST2_CACHE_CAPACITY))
 
-    for (int i = NUM_OF_TEST2_RECORDS - TEST2_CACHE_CAPACITY; i < NUM_OF_TEST2_RECORDS; ++i) {
-        EXPECT_TRUE(cache_lru.exists(i));
-        EXPECT_EQ(i, cache_lru.get(i));
-    }
+    EXPECT_RANGE_FULLY_IN_CACHE(cache_lru, (NUM_OF_TEST2_RECORDS - TEST2_CACHE_CAPACITY), NUM_OF_TEST2_RECORDS, 1)
 
     size_t size = cache_lru.size();
     EXPECT_EQ(TEST2_CACHE_CAPACITY, size);
 }
 
+TEST(CacheTest1, ChangeCacheCapacity) {
+    zim::lru_cache<int, int> cache_lru(TEST2_CACHE_CAPACITY);
+
+    for (int i = 0; i < NUM_OF_TEST2_RECORDS; ++i) {
+        cache_lru.put(i, i);
+    }
+
+    EXPECT_EQ(TEST2_CACHE_CAPACITY, cache_lru.size());
+    EXPECT_RANGE_MISSING_FROM_CACHE(cache_lru, 0, (NUM_OF_TEST2_RECORDS - TEST2_CACHE_CAPACITY))
+    EXPECT_RANGE_FULLY_IN_CACHE(cache_lru, (NUM_OF_TEST2_RECORDS - TEST2_CACHE_CAPACITY), NUM_OF_TEST2_RECORDS, 1)
+
+    cache_lru.setMaxSize(TEST2_CACHE_CAPACITY_SMALL);
+    EXPECT_EQ(TEST2_CACHE_CAPACITY_SMALL, cache_lru.size());
+    EXPECT_RANGE_MISSING_FROM_CACHE(cache_lru, 0, (NUM_OF_TEST2_RECORDS - TEST2_CACHE_CAPACITY_SMALL))
+    EXPECT_RANGE_FULLY_IN_CACHE(cache_lru, (NUM_OF_TEST2_RECORDS - TEST2_CACHE_CAPACITY_SMALL), NUM_OF_TEST2_RECORDS, 1)
+
+    cache_lru.setMaxSize(TEST2_CACHE_CAPACITY);
+    for (int i = 0; i < NUM_OF_TEST2_RECORDS; ++i) {
+        cache_lru.put(i, 1000*i);
+    }
+    EXPECT_EQ(TEST2_CACHE_CAPACITY, cache_lru.size());
+    EXPECT_RANGE_MISSING_FROM_CACHE(cache_lru, 0, (NUM_OF_TEST2_RECORDS - TEST2_CACHE_CAPACITY))
+    EXPECT_RANGE_FULLY_IN_CACHE(cache_lru, (NUM_OF_TEST2_RECORDS - TEST2_CACHE_CAPACITY), NUM_OF_TEST2_RECORDS, 1000)
+}
+
 TEST(ConcurrentCacheTest, handleException) {
     zim::ConcurrentCache<int, int> cache(1);
     auto val = cache.getOrPut(7, []() { return 777; });
